"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = getData;
var _getLocationUrl = _interopRequireDefault(require("../../getLocationUrl.js"));
var _index = require("../../router/index.js");
var _shouldSkipLoadForNavigation = _interopRequireDefault(require("./shouldSkipLoadForNavigation.js"));
var _load = _interopRequireDefault(require("./load.js"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function getData(_ref) {
  var routes = _ref.routes,
    routePosition = _ref.routePosition,
    params = _ref.params,
    dispatch = _ref.dispatch,
    getState = _ref.getState,
    codeSplit = _ref.codeSplit,
    server = _ref.server,
    onError = _ref.onError,
    getLocale = _ref.getLocale,
    getCookie = _ref.getCookie,
    stash = _ref.stash;
  if (!server) {
    if (window._ReactPages_Page_SkipDataLoad || window._ReactPages_Page_SkipDataLoadOnNavigation || window._ReactPages_Page_HotReloadInProgress) {
      if (routePosition === 'leaf') {
        // Reset "skip `load`" flag that was previously set
        // by `pushLocation()` or `replaceLocation()`.
        if (window._ReactPages_Page_SkipDataLoadOnNavigation) {
          delete window._ReactPages_Page_SkipDataLoadOnNavigation;
        }
      }
      return;
    }
  }
  var _getLocations = getLocations(getState()),
    location = _getLocations.location,
    previousLocation = _getLocations.previousLocation;
  var isInitialClientSidePageLoad = !server && !previousLocation;
  // A workaround for `found` router bug:
  // https://github.com/4Catalyzer/found/issues/239
  // Prevent executing `load`s on "anchor" link click.
  if (!server && !isInitialClientSidePageLoad) {
    if ((0, _shouldSkipLoadForNavigation["default"])(previousLocation, location)) {
      return;
    }
  }
  // Execute `load`s.
  return (0, _load["default"])({
    location: location,
    previousLocation: isInitialClientSidePageLoad ? undefined : previousLocation,
    routerArgs: {
      routes: (0, _index.getMatchedRoutes)(getState(), routes),
      routeIndices: (0, _index.getMatchedRoutesIndices)(getState()),
      routeParams: (0, _index.getMatchedRoutesParams)(getState()),
      params: (0, _index.getRouteParams)(getState())
    },
    routePosition: routePosition,
    codeSplit: codeSplit,
    server: server,
    getCookie: getCookie,
    getLocale: getLocale,
    dispatch: dispatch,
    useSelector: function useSelector(getter) {
      return getter(getState());
    }
  }).then(function (result) {
    // Anything that gets returned from this `getData()` function
    // gets passed to the page component as a `data` property.
    switch (routePosition) {
      case 'root':
        stash.setRootComponentProps(result && result.props);
        break;
      case 'leaf':
        stash.setPageComponentProps(result && result.props);
        break;
      default:
        throw new Error("[react-pages] Unsupported route position: \"".concat(routePosition, "\""));
    }
    return result;
  }, function (error) {
    // Possibly handle the error (for example, redirect to an error page).
    if (!(error instanceof _index.RedirectException)) {
      if (onError) {
        onError(error, {
          location: location,
          url: (0, _getLocationUrl["default"])(location),
          // Using `redirect` instead of `goto` here
          // so that the user can't go "Back" to the page being loaded
          // in case of an error because it would be in inconsistent state
          // due to `load` being interrupted.
          redirect: function redirect(to) {
            // `to` can be a `string`.
            //
            // 307 Temporary Redirect
            // https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/307
            // 308 Permanent Redirect
            // https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/308
            //
            var redirectStatusCode = undefined;
            throw new _index.RedirectException(to, redirectStatusCode);
          },
          dispatch: dispatch,
          useSelector: function useSelector(getter) {
            return getter(getState());
          },
          server: server
        });
      }
    }
    throw error;
  });
}
function getLocations(state) {
  var server = typeof window === 'undefined';
  return {
    location: (0, _index.getMatchedLocation)(state),
    previousLocation: server || !window._ReactPages_RouterHasFinishedInitialRender ? undefined : (0, _index.getMatchedLocationThatHasBeenLoaded)(state)
  };
}
//# sourceMappingURL=getData.js.map